#!/bin/csh -f
#
#
set PROGRAM = "Information Collector for ActiveSupport"
set VERSION = "1.0"
set COPYRIGHT = "Copyright (c) 2000 Trend Mirco Incorporated. All right reserved."

set ISCAN   = "/etc/iscan"
set PKGPATH = ${ISCAN}/pkg
set INIFILE = ${ISCAN}/intscan.ini

set GETENG  = ${ISCAN}/geteng
set GETCONF = ${ISCAN}/getconf
set ENVIRON = "./ASenviron.csh"

set TMPDIR  = "tmp"
set RANDOM  = $$
set ARCDIR  = ${TMPDIR}/AS-${RANDOM}
set DATE    = `date +%Y%m%d-%H%M%S`

#
# Header
#
echo ""
echo "$PROGRAM Version ${VERSION}"
echo ""

if ( ! -d ${ISCAN} ) then
  echo "    ALERT: InterScan VirusWall not found. (${ISCAN})"
  echo ""
  exit -1
endif

#
echo "Create archive directory ... "
#
if ( -d ${ARCDIR} ) then
  rm -rf ${ARCDIR}
endif
mkdir -p ${ARCDIR}

#
echo "Get Environment file ... "
#
set ENVFILE = ${ARCDIR}/environ.txt
if ( -x ${ENVIRON} ) then
  ${ENVIRON} >& ${ENVFILE}
else
  echo "Envrionment Collector (${ENVIRON}) not found." >& ${ENVFILE}
endif

set DMESGFILE = ${ARCDIR}/dmesg.txt
switch ( `uname` )
case "AIX":
	alog -t boot -o >& ${DMESGFILE}
	errpt -a >& ${ARCDIR}/errlog.txt
	breaksw
default:
	dmesg >& ${DMESGFILE}
	breaksw
endsw

#
echo "Copy INI file of InterScan ..."
#
ln -s ${INIFILE} ${ARCDIR}

#
echo "Copy LOG files of InterScan ..."
#
if ( -x ${GETCONF} ) then
  foreach s (admin iscvp http ftp smtp Pattern-Update)
    ${GETCONF} $s logfile >> ${ARCDIR}/aa
  end
  set dirlist = `cat ${ARCDIR}/aa | sort -u`
  rm ${ARCDIR}/aa
else
  set dirlist = `grep ^logfile ${INIFILE} | cut -d= -f2 | tr -d ' ' | sort -u`
endif

set c = 1
foreach d ($dirlist)
  echo "    ${c} directory  - ${d} ..."
  set flist = `ls ${d}.????.??.??`
  foreach f ($flist)
    set lname =  ${c}-`basename ${f}`
    echo "        ${f} ..."
    ln -s ${f} ${ARCDIR}/${lname}
  end
  @ c++
end

if ( -f ${PKGPATH}/ISSMTP ) then
  #
  echo "Copy LOG files of original sendmail ..."
  #
  set program = /dev/null

  if ( -x ${GETCONF} ) then
    if ( `${GETCONF} smtp local_mode` == "yes" ) then
      set program = `${GETCONF} smtp command_param | cut -d' ' -f1`
    endif
  else
    set program = `grep ^original ${INIFILE} | tail -1 | cut -d= -f2 | cut -d' ' -f1`
  endif

  if ( -x ${program} ) then
    switch ( `uname` )
    case "SunOS":
      set logfile = "/var/log/syslog"
      breaksw
    default:
      set logfile = `grep ^mail /etc/syslog.conf | tr '\t' ' ' | awk '{print $2}'`
      breaksw
    endsw
    echo "    ${logfile} ..."
    ln -s ${logfile}* ${ARCDIR}
  endif
endif

#
echo "Generate tar ball ..."
#
set COMPRESS = "compress -c"
set SURFIX = "tar.Z"
set GZIP = `which gzip | awk '{print $1}'`
if ( -x ${GZIP} ) then
  set COMPRESS = "${GZIP} -c -9"
  set SURFIX = "tar.gz"
endif
set tarball = ${TMPDIR}/${DATE}.${SURFIX}
(\
  cd ${ARCDIR}; \
  tar cvfh - . | $COMPRESS \
) > ${tarball}
set tsize = `ls -la ${tarball} | awk '{print $5}'`

#
echo "Remove Archive directory ..."
#
rm -rf ${ARCDIR}

# 
echo ""
echo "  Please send ${tarball} (${tsize}bytes) to Tech Support."
echo "  Thank you."
echo ""
echo "    ${COPYRIGHT}"
echo ""

exit 0

