#!/usr/bin/perl
#
# $Id: tmvw-patternupdate.pl,v 1.13 2005/05/02 16:20:06 chris Exp $
#
my $usage = << "END_OF_USAGE";
usage: 
	$0 [ -e errors\@dom.ain ] [ -h ]
	-e errors\@dom.ain : address to send errors to
	-h : help information
	-d : debug mode (up to four -d increment debug level)
	-t : testing (development only)
	-q : quiet (no html output)
 
END_OF_USAGE

use strict;
use LWP;
use Fcntl;
use English;
use POSIX qw(:errno_h);
use Archive::Zip;
use IO::String;

my $intscan_cfg_path = "/etc/iscan";
my $intscan_ini_path = "/etc/iscan/intscan.ini";
my $iscan_user       = "iscan";
my $lock_file = "/etc/iscan/.patupdate";
my $http_method = "GET";
my $logfile = "/var/log/iscan/patternupdate.".filetimestamp();

my $error_mail_addr = undef;
my $debug = 0;
my $testing = 0;
my $quiet = 0; # no html output

doopt(\@ARGV, { 
    # use "-x parameter" 
    -e => sub { $error_mail_addr  = shift @{$_[0]} },
    # use "-x" as a flag
    -h => sub { usage(); exit 0 },
    -d => sub { $debug++ },
    -t => sub { $testing=1 },
	-q => sub { $quiet=1; },
});

logging("started ...");

if ( $testing ) {
	logging("test mode enabled");
	my $testdir =
		"/usr/local/users/support/chris/prog/perl/tmvw-patternupdate";
	$intscan_cfg_path = $testdir;
	$intscan_ini_path = "$testdir/intscan.ini";
	$lock_file        = "$testdir/.patupdate";
	$http_method 	  = "GET";
	$iscan_user       = "chris";
}

my $tmp_file = "vpnfile.tmp";
my $pattern_file_template = "%s/lpt\$vpn.%s";

my $tmvw_cfg_lines = [];
my $tmvw_cfg = {};
my $serv_cfg = {};
my $msg_accu = [];
my $trials_cnt   = 1; # a counter
my $trials_nr_of = 5;

logging("retry count=$trials_nr_of.");

parse_ini_file("FILE",$intscan_ini_path,$tmvw_cfg,$tmvw_cfg_lines);
$debug >= 2 and dump_settings();

# 3.6
#my $download_uri = sprintf "%s?serial=%s&Version=%s&VSAPI=yes",
#	$tmvw_cfg->{"Registration"}->{"download_path"}->{"VAL"},
#	$tmvw_cfg->{"Registration"}->{"Serial"}->{"VAL"},
#	$tmvw_cfg->{"Pattern-Update"}->{"Version"}->{"VAL"};
#my $download_host = $tmvw_cfg->{"Registration"}->{"hostname"}->{"VAL"};
#
#my $download_url = sprintf "http://%s%s",$download_host,$download_uri;

# 3.8
my $server_ini_url = $tmvw_cfg->{"Registration"}->{"hostname"}->{"VAL"} . "/server.ini";
$debug and print "URL: ",$server_ini_url,"\n";

my $server_ini;
for ( $trials_cnt=1; $trials_cnt <= $trials_nr_of; $trials_cnt++ ) {
	logging("Try $trials_cnt...");

	($server_ini) = server_ini_download($server_ini_url);

	last if $server_ini;

	my $wait = 10 * rand;  
	logging("connect failed, waiting $wait s");
	sleep $wait;
}
if ( ! $server_ini ) {
	logging("could not receive server.ini");
	exit_with_message(1);
}
parse_ini_file("SCALAR",$server_ini,$serv_cfg);

$debug and printf "Local  VERSION: %s\n",$tmvw_cfg->{"Pattern-Update"}->{"Version"}->{"VAL"};
$debug and printf "Remote VERSION: %s\n",$serv_cfg->{"PATTERN"}->{"Version"}->{"VAL"};

my $pattern_version = $serv_cfg->{"PATTERN"}->{"Version"}->{"VAL"};
# local and remote version are equal? local pattern file really exists? no update needed
if ( $pattern_version == $tmvw_cfg->{"Pattern-Update"}->{"Version"}->{"VAL"} ) {
	my ($patten_file_rc,$pattern_file_msg) = check_virus_pattern_file (
		$tmvw_cfg->{"Scan-configuration"}->{"pattern_path"}->{"VAL"},
		$pattern_file_template,
		$pattern_version);

	if ( $patten_file_rc == 0 ) {
		$debug and print $pattern_file_msg."\n";
		$debug and print "no update necessary\n";
		logging("pattern file is up to date.");
		$quiet || html_message_up2date();
		exit 0;
	} elsif ( $patten_file_rc == 1 ) {
		$debug and print $pattern_file_msg."\n";
		$debug and print "just getting pattern file\n";
		logging("pattern file is missing, getting new one.");
	} elsif ( $patten_file_rc == 2 ) {
		log_message_and_exit("ERROR: $pattern_file_msg");
	} else {
		add_message("ERROR: invalid returncode '$patten_file_rc' from check_virus_pattern_file()");
		exit_with_message(1);
	}

}
my ($download_file,$download_size) = split /,/, $serv_cfg->{"PATTERN"}->{"Path_VSAPI"}->{"VAL"};
my $download_url = $tmvw_cfg->{"Registration"}->{"hostname"}->{"VAL"} . 
					"/" .  $download_file;

$debug and print "VERSION: ",$pattern_version,"\n";
$debug and print "URL: ",$download_url,"\n";

lock_file();

my $virus_pattern_zip; 
for ( $trials_cnt=1; $trials_cnt <= $trials_nr_of; $trials_cnt++ ) {
	($virus_pattern_zip) = pattern_download($download_url,$pattern_version);

	last if $virus_pattern_zip;

	sleep 10 * rand;  
}
$debug and print "SIZE: ",length($virus_pattern_zip),"\n";
if ( ! $virus_pattern_zip ) {
	exit_with_message(1);
}

my $virus_pattern = unpack_pattern_zip(\$virus_pattern_zip);

write_virus_pattern_file ( 
	$tmvw_cfg->{"Scan-configuration"}->{"pattern_path"}->{"VAL"},
	$tmp_file,
	$pattern_file_template,
	$virus_pattern,
	$pattern_version);

change_ini_file_data (
	$tmvw_cfg_lines,
	$tmvw_cfg,
	"Pattern-Update",
	"Version",
	$pattern_version);

write_intscan_ini_file ( 
	$intscan_ini_path,
	$tmvw_cfg_lines,
	$pattern_version );

unlock_file();

remove_old_pattern_files( $tmvw_cfg->{"admin"}->{"num_pattern_kept"}->{"VAL"} );

inform_services();

exit_with_message(0);

########################
# subroutine collection
########################

sub parse_ini_file {
	my ($type,$file,$ini_file_struct,$ini_file_lines) = @_;
	my $section = "";
	my @Lines;

	if ( $type eq "FILE" ) {
		open F,$file or do {
			add_message("ERROR: Can't open '$intscan_ini_path': $!");
			exit_with_message(1);
		}; 
		@Lines = <F>;
		close F;
	} elsif ( $type eq "SCALAR" ) {
		@Lines = split /\n/,$file;
	} else {
		add_message("ERROR: internal problem: parse_ini_file():type:$type");
		exit_with_message(1);
	}

	$debug >= 3 and print "PIF: ";
	for (my $i=0;$i <= $#Lines; $i++) {
		my $line = $Lines[$i];
		push @$ini_file_lines, $line;
		next if $line =~ /^[#;]/;
		next if $line =~ /^\s*$/;
		if ( my($varname,$value) = $line =~ /^([A-Za-z0-9_.]+?)\s*=\s*(.*?)\s+$/ ) {
			$ini_file_struct->{$section}->{$varname}->{VAL}  = $value;
			$ini_file_struct->{$section}->{$varname}->{LINE} = $i;
			$ini_file_struct->{$section}->{$varname}->{CHG}  = 0;
			$debug >= 3 and print "$section $varname ($value, $.)\n";
			$debug >= 3 and print $ini_file_struct->{$section}->{$varname}->{VAL},"\n";
			$debug >= 3 and print $tmvw_cfg->{$section}->{$varname}->{VAL},"\n";
			$debug >= 3 and print $ini_file_struct->{$section}->{$varname}->{LINE},", ";
		} elsif ( ($section) = $line =~ /^\[(\S+)\]/ ) {
			$debug >= 3 and print "$section\n";
		} else {
			add_message("WARNING: undefined line: $line");
		}
	}
	$debug and print "*\n";
}

sub get_value_from_ini_file_content {
	my ($var_name,$section,$cfg_file_content) = @_;
	my ( $value ) = $cfg_file_content =~ m/^${var_name}=(.*?)\s+$/m; 
	return $value;
}

sub dump_settings {
	print "Scan-configuration pattern_path: ",
		$tmvw_cfg->{"Scan-configuration"}->{"pattern_path"}->{"VAL"},"\n";
	print "Scan-configuration license : ",
		$tmvw_cfg->{"Scan-configuration"}->{"license"}->{"VAL"},"\n";
	print "Scan-configuration update: ",
		$tmvw_cfg->{"Scan-configuration"}->{"update"}->{"VAL"},"\n";
	print "Scan-configuration update_interval: ",
		$tmvw_cfg->{"Scan-configuration"}->{"update_interval"}->{"VAL"},"\n";

	print "admin num_pattern_kept: ",
		$tmvw_cfg->{"admin"}->{"num_pattern_kept"}->{"VAL"},"\n";

	print "Pattern-Update Version: ",
		$tmvw_cfg->{"Pattern-Update"}->{"Version"}->{"VAL"},"\n";
	print "Pattern-Update Frequency: ",
		$tmvw_cfg->{"Pattern-Update"}->{"Frequency"}->{"VAL"},"\n";
	print "Pattern-Update admin_addr: ",
		$tmvw_cfg->{"Pattern-Update"}->{"admin_addr"}->{"VAL"},"\n";
	print "Pattern-Update Subject: ",
		$tmvw_cfg->{"Pattern-Update"}->{"Subject"}->{"VAL"},"\n";
	print "Pattern-Update logfile: ", 
		$tmvw_cfg->{"Pattern-Update"}->{"logfile"}->{"VAL"},"\n";

	print "Registration hostname: ", 
		$tmvw_cfg->{"Registration"}->{"hostname"}->{"VAL"},"\n";
	print "Registration download_path: ",
		$tmvw_cfg->{"Registration"}->{"download_path"}->{"VAL"},"\n";
	print "Registration Serial: ", 
		$tmvw_cfg->{"Registration"}->{"Serial"}->{"VAL"},"\n";
}

sub server_ini_download {
	my ($url) = @_;

	logging("server.ini server connect.");

	my $ua = new LWP::UserAgent;
	my $req = new HTTP::Request $http_method => $url;
	my $res = $ua->request($req);
	if ($res->is_success) {
		if ( length $res->content != $res->header("Content-length") ) {
			logging("server.ini file size is unmatched.");
			logging("server.ini server disconnect.");
			add_message("ERROR: server.ini file size is unmatched.");
			return 0;
		} else {
			logging("server.ini download complete.");
			logging("server.ini server disconnect.");
			add_message("INFO: Download of server.ini file completed.");
			return $res->content;
		}
	} else {
		logging("HTTP response code error : ".$res->code);
		logging("server.ini server disconnect.");
		add_message("ERROR: HTTP Response code error : ".$res->code);
	}
	return 0;
}

sub pattern_download {
	my ($url,$pattern_version) = @_;

	logging("Pattern server connect.");

	my $ua = new LWP::UserAgent;
	my $req = new HTTP::Request $http_method => $url;
	my $res = $ua->request($req);
	if ($res->is_success) {
		if ( length $res->content != $res->header("Content-length") ) {
			logging("Pattern file size is unmatched.");
			logging("Pattern server disconnect.");
			add_message("ERROR: Pattern file size is unmatch.");
			return 0;
		} else {
			logging("Pattern download complete.(No.$pattern_version)");
			logging("Pattern server disconnect.");
			add_message("SUCCESS: Download of virus pattern file completed." .
		            "(New pattern No. $pattern_version)");
			return $res->content;
		}
	} else {
		logging("HTTP response code error : ".$res->code);
		logging("Pattern server disconnect.");
		add_message("ERROR: HTTP Response code error : ".$res->code);
	}
	return 0;
}

sub write_intscan_ini_file {
	my ($file,$tmvw_cfg_lines,$pattern_version) = @_;
	my $new_file = "$file.tmp";
	my $bak_file = "$file.sav";
	my (undef,undef,$uid,$gid) = getpwnam($iscan_user) 
		or log_message_and_exit("Can't getpwnam() on '$iscan_user': $!");

	open P,">$new_file"     
		or log_message_and_exit("Can't open file '$new_file': $!");
	print P @$tmvw_cfg_lines 
		or log_message_and_exit("Can't write file '$logfile': $!");
	close P
		or log_message_and_exit("Can't close file '$logfile': $!");
	chown $uid, $gid, $new_file 
		or log_message_and_exit("Can't chown($uid,$gid) file '$new_file': $!");

	unlink $bak_file if -e $bak_file;
	rename $file,$bak_file 
		or log_message_and_exit("Can't rename '$file' to '$bak_file': $!");
	rename $new_file,$file 
		or log_message_and_exit("Can't rename '$new_file' to '$file': $!");
}

sub check_virus_pattern_file {
	my ($dir,$pattern_file_template,$pattern_version) = @_;
	my $pattern_file = sprintf($pattern_file_template,$dir,$pattern_version);

	$debug >= 3 and printf "pattern_file: '%s'\n",$pattern_file;

	(   -f $pattern_file ) && do { return (0,"'$pattern_file' does exist") };
	( ! -e $pattern_file ) && do { return (1,"'$pattern_file' does not exist") };
	( ! -f $pattern_file ) && do { return (2,"'$pattern_file' does exist but is no plain file") };
}

sub write_virus_pattern_file {
	my ($dir,$tmp_file,$pattern_file_template,$content,$pattern_version) = @_;
	my $path_file = "$dir/$tmp_file";
	my $pattern_file = sprintf($pattern_file_template,$dir,$pattern_version);

	open P,">$path_file" 
		or log_message_and_exit("Can't open file '$path_file': $!");
	print P $content     
		or log_message_and_exit("Can't write file '$path_file': $!");
	close P              
		or log_message_and_exit("Can't close file '$path_file': $!");
	$debug >= 4 and debug_file_stats($path_file);

	rename $path_file,$pattern_file 
		or log_message_and_exit("Can't rename '$path_file' to '$pattern_file': $!");
	$debug >= 4 and debug_file_stats($pattern_file);
}

sub lock_file {
	# open the file for writing, creating a new file which must not
	# previously exist:
	sysopen P,"$lock_file",O_WRONLY|O_CREAT|O_EXCL or do {
		my $e = $!+0;
		$debug and print "errno: $e\n";
		($e == EEXIST ) && add_message("ERROR: Other process is updating pattern file.");
		add_message("ERROR: $e");
		exit_with_message(1);
	};
	close P;
}

sub unlock_file {
	unlink $lock_file or die "Can't unlink '$lock_file': $!";
}

sub change_ini_file_data {
	my ( $ini_file_lines, $ini_file_struct, $section,$varname,$value ) = @_;
	$debug and printf "%s->%s->VAL = %s -> %s\n",$section,$varname,
		$ini_file_struct->{$section}->{$varname}->{VAL}, $value;
	$debug and printf "LINE %s: %s->",$ini_file_struct->{$section}->{$varname}->{LINE},
		$ini_file_lines->[ $ini_file_struct->{$section}->{$varname}->{LINE} ];

	$ini_file_struct->{$section}->{$varname}->{VAL} = $value;
	$ini_file_struct->{$section}->{$varname}->{CHG} = 1;
	$ini_file_lines->[ $ini_file_struct->{$section}->{$varname}->{LINE} ] = 
		sprintf "%s=%s\n",$varname,$value;

	$debug and printf "LINE: %s",$ini_file_lines->[ $ini_file_struct->{$section}->{$varname}->{LINE} ];
	# print $ini_file_lines->[ $ini_file_struct->{$section}->{$varname}->{LINE} ];
}

sub send_mail {
	my ($rcpt_group,@Message) = @_;
	my $rcpt;
	if ( $rcpt_group eq "all" ) { 
		$rcpt = $tmvw_cfg->{"Pattern-Update"}->{"admin_addr"}->{"VAL"};
	} elsif ( $rcpt_group eq "error" and defined $error_mail_addr ) {
		$rcpt = $error_mail_addr;
	}

	logging("delivering notification to ".$tmvw_cfg->{"Pattern-Update"}->{"admin_addr"}->{"VAL"});
	my @Status_mail = (
		"From: ".$tmvw_cfg->{"Pattern-Update"}->{"admin_addr"}->{"VAL"},
		"To: ".$rcpt,
		"Subject: ".$tmvw_cfg->{"Pattern-Update"}->{"Subject"}->{"VAL"},
		"",
		"Notification: Automatic Virus Pattern Update",
		"",
		@Message,
		""
	);

	# see "Safe Pipe Opens" in man perlipc
	my $pid = open(M, "|-");
	# $SIG{ALRM} = sub { die "whoops, /usr/sbin/sendmail pipe broke" };
	if ($pid) {  # parent
		print M join "\n",@Status_mail;
		close M;
	} else {     # child
		($EUID, $EGID) = ($UID, $GID);
		$debug and print "MAILTO: ",$tmvw_cfg->{"Pattern-Update"}->{"admin_addr"}->{"VAL"},"\n";
		exec qw(/usr/sbin/sendmail -t -f),$tmvw_cfg->{"Pattern-Update"}->{"admin_addr"}->{"VAL"},
			|| die "can't exec /usr/sbin/sendmail: $!";
		# NOTREACHED
	}
}

sub timestamp {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	return sprintf "%02d/%02d/%04d %02d:%02d:%02d",$mon+1,$mday,$year+1900,$hour,$min,$sec;
}

sub filetimestamp {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	return sprintf "%04d.%02d.%02d", $year+1900, $mon+1, $mday;
}

sub add_message {
	push @$msg_accu,sprintf "[%s] Try %-2s:",timestamp, $trials_cnt;
	push @$msg_accu,$_[0];
	push @$msg_accu,"";
}

sub exit_with_message {
	my ($stat) = @_;
	if ( -f $lock_file ) {
		unlock_file();
	}
	$debug and print join "\n",@$msg_accu;
	if ($stat != 0) {
		$quiet || html_message_fail($msg_accu);
		send_mail("error",@$msg_accu);
	} else {
		$quiet || html_message_ok($pattern_version);
	}
	send_mail("all",@$msg_accu);
	exit;
}

sub unpack_pattern_zip {
	my ( $zip_content_ref ) = @_;

	my $zip = Archive::Zip->new();
	$zip->readFromFileHandle( IO::String->new($zip_content_ref) );
	
	my @MemberNames = $zip->memberNames();
	$debug and print "ZIPCONTENT: @MemberNames\n";
	if ( scalar @MemberNames != 1 || $MemberNames[0] != /^lpt\$vpn\.\d{3,4}$/ ) {
		add_message("ERROR: zip file content illegal:unpack_virus_zip():@MemberNames");
		exit_with_message(1);
	}
	return $zip->contents( $MemberNames[0] );
}

sub remove_old_pattern_files {
	my ($files_to_keep) = @_;

	# set default files to keep to 3
	if ( ! $files_to_keep ) { 
		$files_to_keep = 3;
	}

	opendir D,$intscan_cfg_path or do {
		add_message("ERROR: can't open '$intscan_cfg_path': $!");
		exit_with_message(1);
	};
	#
	# read directory, filter lpt$vpn.* named FILES and sort reverse by pattern file mtime.
	#
	my @Files = sort { get_pattern_file_mtime("$intscan_cfg_path/$b") <=> get_pattern_file_mtime("$intscan_cfg_path/$a") }
	            grep { /^lpt\$vpn\.\d+$/ && -f "$intscan_cfg_path/$_" }
				readdir(D);
	closedir D;
	$debug >= 4 and print "pattern files: @Files\n";
	# remove the newest files from the list
	splice @Files,0,$files_to_keep;
	$debug >= 4 and print "kept files: $files_to_keep\n";
	# delete the remaining ones
	for my $f (@Files) {
		$debug >= 4 and print "unlinking $intscan_cfg_path/$f\n";
		unlink "$intscan_cfg_path/$f" or 
			add_message("WARNING: could not remove '$intscan_cfg_path/$f': $!");
	}
}

sub get_pattern_file_seqnr {
	return ( $_[0] =~ m/^lpt\$vpn\.(\d+)$/ )[0]
}

sub get_pattern_file_mtime {
	my ($file) = @_;
	my $mtime = (stat $file)[9] or do {
		add_message("ERROR: can't stat '$file': $!");
		exit_with_message(1);
	};
	return ($mtime);
}

sub inform_services {
	#
	# todo: 
	# - ausfuehrung ueberwachen
	# - meldungen erzeugen
	# - nur killen, was im ini-file auch aktiviert ist.
	# 
	my $rc;

	logging("reflash daemon");

	$rc = system qw( /etc/iscan/daemonhup.sh sendmail ) and do {
		my $msg = sprintf("fail to reflash daemon(sendmail).(system=%d)",$rc);
		logging($msg);
	};

	$rc = system qw( /etc/iscan/daemonhup.sh isftpd ) and do {
		my $msg = sprintf("fail to reflash daemon(isftpd).(system=%d)",$rc);
		logging($msg);
	};

	$rc = system qw( /etc/iscan/daemonhup.sh ishttpd ) and do {
		my $msg = sprintf("fail to reflash daemon(ishttpd).(system=%d)",$rc);
		logging($msg);
	};

	$rc = system qw( /etc/iscan/daemonhup.sh iscvp ) and do {
		my $msg = sprintf("fail to reflash daemon(iscvp).(system=%d)",$rc);
		logging($msg);
	};
}

sub logging {
	open L, ">>".$logfile or do {
			add_message("ERROR: Can't open log file '$logfile': $!");
			exit_with_message(1);
	};
	printf L "%s ptn[%s]: %s\n", timestamp(), $$, $_[0] or do {
			add_message("ERROR: Can't write log file '$logfile': $!");
			exit_with_message(1);
	};
	close L or do {
			add_message("ERROR: Can't close log file '$logfile': $!");
			exit_with_message(1);
	};
}

sub log_message_and_exit {
	logging($_[0]);
	add_message("ERROR: $_[0]");
	exit_with_message(1);
}

sub html_message_up2date {
	print join "\n",
		'Content-Type: text/html',
		'',
		'<TITLE>Update Virus Pattern Now</TITLE>',
		'<HTML>',
		'<BODY BGCOLOR=#FFFFFF>',
		'<H2>Pattern up to date.</H2><HR size=1>',
		'',
		'<CENTER><EM>',
		'',
		'Find out more about virus protection at',
		'',
		'<A HREF="http://www.antivirus.com">',
		' ',
		'http://www.antivirus.com </A>',
		'',
		'<BR>Copyright &#169; 1996-2003 Trend Micro Incorporated </EM>',
		'',
		'</CENTER>',
		'',
		'</BODY> </HTML>',
		'';
}

sub html_message_ok {
	my ($version) = @_;
	print join "\n",
		'Content-Type: text/html',
		'',
		'<TITLE>Update Virus Pattern Now</TITLE>',
		'<HTML>',
		'<BODY BGCOLOR=#FFFFFF>',
		"<H2>New pattern version $version.</H2><HR size=1>",
		'',
		'<CENTER><EM>',
		'',
		'Find out more about virus protection at',
		'',
		'<A HREF="http://www.antivirus.com">',
		' ',
		'http://www.antivirus.com </A>',
		'',
		'<BR>Copyright &#169; 1996-2003 Trend Micro Incorporated </EM>',
		'',
		'</CENTER>',
		'',
		'</BODY> </HTML>',
		'';
}

sub html_message_fail {
	my ($reason) = @_;
	print join "\n",
		'Content-Type: text/html',
		'',
		'<TITLE>Update Virus Pattern Now</TITLE>',
		'<HTML>',
		'<BODY BGCOLOR=#FFFFFF>',
		'<H2>Pattern update failed.</H2><HR size=1>',
		'<PRE>',
		@$reason,
		'</PRE>',
		'<CENTER><EM>',
		'',
		'Find out more about virus protection at',
		'',
		'<A HREF="http://www.antivirus.com">',
		' ',
		'http://www.antivirus.com </A>',
		'',
		'<BR>Copyright &#169; 1996-2003 Trend Micro Incorporated </EM>',
		'',
		'</CENTER>',
		'',
		'</BODY> </HTML>',
		'';
}

#
# process command line arguments
# usage:
#
# doopt(\@ARGV, { 
#     # use "-x parameter" 
#     -o => sub { $outFileBase  = shift @{$_[0]} },
#     # use "-x" as a flag
#     -f => sub { $flag         = 1              },
# });
#

sub doopt {
    my ($argvRef, $optRef ) = @_;
    my @ArgBak = ();
    while ( my $arg = shift @$argvRef ) {
        if ( defined $optRef->{$arg} ) {
            $optRef->{$arg}->($argvRef);
        } else {
            push @ArgBak, $arg;
        }
    }
    @$argvRef = @ArgBak;
}

sub usage {
	print $usage;
}

sub debug_file_stats {
	my ($filename) = @_;
	my($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
		$atime,$mtime,$ctime,$blksize,$blocks) = stat($filename);
		printf "file $filename status:\n".
			   "  dev: %s \n".
			   "  ino: %s \n".
			   "  mode: %07o \n".
			   "  nlink: %s ".
			   "  u/gid: %s/%s \n".
			   "  rdev: %s \n".
			   "  size: %s \n".
			   "  atime: %s \n".
			   "  mtime: %s \n".
			   "  ctime %s \n".
			   "  blksz: %s ".
			   "  blocks %s \n",
			$dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
			scalar localtime $atime,
			scalar localtime $mtime,
			scalar localtime $ctime,$blksize,$blocks;
}
