#!/bin/sh


BELL=`tput bel`

if [ -f /etc/iscan/pkg/ISVWROOT ]
then
	isuxroot=`cat /etc/iscan/pkg/ISVWROOT`
fi

empkg=EMpkg
INTERSCAN_HTTPD_SERVICE=0
UpgradeBackupConfig='/tmp/EMInstallFiles/upgrade'
CheckVersion='/tmp/EMInstallFiles/checkver'

#---------------------------------------------------------
#  Set platform dependent variables
#---------------------------------------------------------
SetPlatformDependVar()
{
	INTERSCAN_HTTPD_SERVICE=/etc/rc.d/rc2.d/K99IScanHttpd

	IStmp=`ps -ef | grep '/etc/iscan/sendmail' | grep -v grep`
	
}

#---------------------------------------------------------
#  check root authority
#---------------------------------------------------------
CheckRoot()
{
	uid=`id|sed -e 's/uid=/ /g' -e 's/(/ /g'|awk '{print $1}'`
	
	if [ $uid -ne 0 ]; then
        printf "\t\tSorry, only users logged on using an account with root-level rights can install/remove this package.\n\n" ;
        return 1
    else
    	return 0
	fi
}

#=========================================================
# clean all temporary files
#=========================================================
Clean_Temp()
{
    #printf "Clean temporary files...\n"
    
    if [ -d /tmp/EMInstallFiles ]
    then
    	rm -rf /tmp/EMInstallFiles
    fi
    
    if [ -f /tmp/Install.tar ]
    then
    	rm /tmp/Install.tar
    fi
    	
    #printf "Setup script terminate\n"
    exit 0
}

#---------------------------------------------------------
#  print header
#---------------------------------------------------------
Header()
{
    tput clear
    printf "\n"
    printf "\t\tTrend InterScan eManager for UNIX 3.8\n"
    printf "\n"
    printf " ------------------------------------------------------------------------------\n"
    printf "\n"

    return
}

#---------------------------------------------------------
#  InputSerialNum
#---------------------------------------------------------
InputSerialNum()
{
	Header
	printf "\n"
	printf "\t\tPlease enter your InterScan eManager serial number.\n"
	printf "\t\tNote: If the serial number field is left blank,\n"
	printf "\t\tthe 30-day trial version will be installed.\n"
	
	printf "\n"
	printf "\n"
	printf "\t  \tSerial No:[                        ]\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b";read SNRes
	
	#Check serial number.
	if [ $SNRes ]
	then
		RightKey=`/tmp/EMInstallFiles/keycheck chkkey $SNRes`
	else
		RightKey=`/tmp/EMInstallFiles/keycheck chkkey Hello_Kitty`
	fi
	
	if [ $RightKey -eq -1 ]
	then
		printf "\n"
		printf "\t\tInvalid serial number, or no serial number entered.\n"
		printf "\t\tThe 30-day trial version will be installed.\n"
		printf "\n"
		printf '\t\tPress "Enter" to continue...';read nouse
		/tmp/EMInstallFiles/WriteSNtoFile 'trial'
	elif [ $RightKey -eq 0 -o $RightKey -eq 2 ]
	then
		printf "\n"
		printf "\t\tRegistration complete.\n"
		printf "\n"
		printf '\t\tPress "Enter" to continue...';read nouse
		/tmp/EMInstallFiles/WriteSNtoFile 'serial' $SNRes
		
	fi
	
	
	return 0
	
}

#---------------------------------------------------------
#  Get and Set language on host
#---------------------------------------------------------
SetHostLanguage()
{
	if [ "$LANG" != "" ]; then
		HostLang=$LANG
	elif [ "$LC_ALL" != "" ]; then
		HostLang=$LC_ALL
	else
		HostLang=C
	fi

	mbFlag=`grep -c "^[Hh]ost[Ll]anguage=" /tmp/EMInstallFiles/contscan.ini`
	if [ $mbFlag -eq 1 ]; then
		cat /tmp/EMInstallFiles/contscan.ini | sed -e "s/^[Hh]ost[Ll]anguage=C/hostlanguage=$HostLang/" > /tmp/EMInstallFiles/temp.ini
	else
		sed -n -e '1p' /tmp/EMInstallFiles/contscan.ini > /tmp/EMInstallFiles/temp.ini
		echo "hostlanguage=$HostLang" >> /tmp/EMInstallFiles/temp.ini
		sed -e '1d' /tmp/EMInstallFiles/contscan.ini >> /tmp/EMInstallFiles/temp.ini
	fi
	mv /tmp/EMInstallFiles/temp.ini /tmp/EMInstallFiles/contscan.ini
}

#---------------------------------------------------------
#  AddPackage
#---------------------------------------------------------
AddPackage()
{
	#printf "$isuxroot\n"
	#printf "$empkg\n"
	tarfiledir=`pwd`
	
	cd $isuxroot
	cd Plug-Ins
	
	if [ -d EM ]
	then 
	    rm -rf EM
	fi
	
	umask 022
	
	tar xovf $tarfiledir/EM.tar

}

#---------------------------------------------------------
#  AddAliasFor36
#---------------------------------------------------------
AddAliasFor36()
{
ed $isuxroot/ISADMIN/IScan.adm/conf/httpd.conf << T_EOF > /dev/null
i
Alias /e-images/ $isuxroot/Plug-Ins/EM/cgi/e-images/
Alias /eManager/ $isuxroot/Plug-Ins/EM/cgi/Pages/
Alias /eJava/ $isuxroot/Plug-Ins/EM/cgi/Java/
.
w
q
T_EOF
}

#---------------------------------------------------------
#  AddAliasFor37
#---------------------------------------------------------
AddAliasFor37()
{
ed $isuxroot/ISADMIN/IScan.adm/conf/httpd.conf << T_EOF > /dev/null
/Alias
i
Alias /e-images/ $isuxroot/Plug-Ins/EM/cgi/e-images/
Alias /eManager/ $isuxroot/Plug-Ins/EM/cgi/Pages/
Alias /eJava/ $isuxroot/Plug-Ins/EM/cgi/Java/
.
w
q
T_EOF
}

#---------------------------------------------------------
#  MakeFile
#---------------------------------------------------------
MakeFile()
{
	ln -s $isuxroot/Plug-Ins/EM/cgi $isuxroot/ISADMIN/IScan.adm/cgi-bin/EM
		
	###Web Server Configuration.	
	#AliasLine='Alias /eManager/ '$isuxroot'/Plug-Ins/EM/cgi/Pages/'
	#echo $AliasLine >> $isuxroot/ISADMIN/IScan.adm/conf/srm.conf
	
	#AliasLine='Alias /e-images/ '$isuxroot'/Plug-Ins/EM/cgi/e-images/'
	#echo $AliasLine >> $isuxroot/ISADMIN/IScan.adm/conf/srm.conf

	#AliasLine='Alias /eJava/ '$isuxroot'/Plug-Ins/EM/cgi/Java/'
	#echo $AliasLine >> $isuxroot/ISADMIN/IScan.adm/conf/srm.conf

	if [ -f $isuxroot/ISADMIN/IScan.adm/cgi-bin/about ]
	then
		isntver=`$isuxroot/ISADMIN/IScan.adm/cgi-bin/about|grep 'SMTP version:'|sed -e 's/.*version: //g' -e 's/-/ /'|awk '{print $1}'`
		#printf $isntver;read nouse
		if [ "$isntver" = "3.6" ]; then
			#printf "\n\nfor 3.6\n\n";read nouse
			AddAliasFor36
		else
			#printf "\n\nnot for 3.6\n\n";read nouse
			AddAliasFor37
		fi
    else
		#printf "\n\nnot for 3.6, no file\n\n";read nouse
		AddAliasFor37
	fi
	
	### Add security check for *.exe 
	#AccessConf=$isuxroot/ISADMIN/IScan.adm/conf/access.conf
	#cat $AccessConf | sed -e 's/\.cgi\$/\.(cgi\|exe)\$/1' > /tmp/EMInstallFiles/access.tmp
	#mv /tmp/EMInstallFiles/access.tmp $AccessConf
	
	$INTERSCAN_HTTPD_SERVICE stop
	$INTERSCAN_HTTPD_SERVICE start
	
	cp /tmp/EMInstallFiles/contscan.ini $isuxroot/Plug-Ins/EM/
	echo $isuxroot > /etc/EMUXROOT
	
	###Create /etc/iscan_plugins/em.dsc
	echo '[plug-in]' > /etc/iscan_plugins/em.dsc
	echo 'name=eManager' >> /etc/iscan_plugins/em.dsc
	echo 'root_path='$isuxroot >> /etc/iscan_plugins/em.dsc
	echo 'shared_lib='$isuxroot'/Plug-Ins/EM/lib/em.so' >> /etc/iscan_plugins/em.dsc
	echo 'cgi_cnfg=eManager/eManager.html' >> /etc/iscan_plugins/em.dsc
	echo 'cgi_rpt=EM/LogDate.cgi' >> /etc/iscan_plugins/em.dsc
	echo 'restore_script='$isuxroot'/Plug-Ins/EM/restore.sh' >> /etc/iscan_plugins/em.dsc
#	echo 'version='$_version >> /etc/iscan_plugins/em.dsc
	
	chmod -R 755 /etc/iscan_plugins
}

#---------------------------------------------------------
#  MakeDir
#---------------------------------------------------------
MakeDir()
{
	mkdir $isuxroot/Plug-Ins/EM/tempdir
}

#---------------------------------------------------------
# PreInstall
#---------------------------------------------------------
PreInstall()
{
	umask 022
	cp Install.tar /tmp;(cd /tmp; tar xovf Install.tar > /dev/null)
	return 0
}

#---------------------------------------------------------
# InstallConfirmation
#---------------------------------------------------------
InstallConfirmation ()
{
	if [ -f /etc/iscan/pkg/ISVWROOT ]
	then
		if [ -f /etc/iscan_plugins/em.dsc ]
		then
			
			emuxroot=`cat /etc/iscan_plugins/em.dsc | grep root_path | sed -e 's/=/ /' | awk '{print $2}'`
			
			if [ -f	"$emuxroot/Plug-Ins/EM/lib/em.so" ]
			then
			
				Header
				printf "\n"
				printf "\n"
				
				printf "\t\teManager is already installed on this system.\n"
				printf "\n"
				printf "\n"
				
				Exit
			fi
		fi
		
		return 0

	else
		Header
		printf "\n"
		printf "\n"
		
		printf "\t\tInterScan E-Mail VirusWall is required to install eManager.\n"
		printf "\n"
		printf "\n"
		
		Exit
	fi
		
}

#---------------------------------------------------------
# SetCronTable
#---------------------------------------------------------
SetCronTable()
{
	crontab -l > /tmp/EMInstallFiles/roottab
	echo '0 2 * * * '$isuxroot'/Plug-Ins/EM/cgi/dailylog' >> /tmp/EMInstallFiles/roottab
	crontab /tmp/EMInstallFiles/roottab
}

#=========================================================
#	UseOldINI 
#=========================================================
UseOldINI ()
{

		if [ -f /etc/iscan_plugins/EM/contscan.ini ]
		then
			
			while true ; do
			Header
	
			printf "\n"
			printf "\n"
			printf "\t\tUse existing eManager rule files?[y,n] ";read KEEP
	
			if [ "$KEEP" = y -o "$KEEP" = Y ]
			then
			
				RestoreConfig
				return
				
			elif [ "$KEEP" = n -o "$KEEP" = N ]
			then
				return
			else
				printf "${BELL}\t\t"
			fi
	
			done
		
		else
			return
		fi
}

#=========================================================
# Install_Process
#=========================================================
Install_Process()
{
	InstallConfirmation
	InputSerialNum

	SetHostLanguage

	AddPackage
	MakeFile
	MakeDir
	SetCronTable

	UseOldINI
	chmod 660 $isuxroot/Plug-Ins/EM/contscan.ini
	chown -R root $isuxroot/Plug-Ins/EM
	chgrp -R iscan  $isuxroot/Plug-Ins/EM
	
	Header
	printf '\t\tInterScan eManager has been installed.\n'
	printf "\n"
	printf '\t\tEnable eManager from the InterScan E-Mail\n'
	printf '\t\t VirusWall console.\n\n'
	
	Exit
}

#---------------------------------------------------------
#  Check if InterScan is running
#---------------------------------------------------------
IsInterScanRunning ()
{
	ISrunning=`echo $IStmp | sed -e 's/ /=/g'`

	if [ $ISrunning ]
	then
		Header
		printf "\n"
		printf "\n"
		
		printf "\t\tPlease stop the InterScan E-Mail VirusWall daemon,\n"
		printf "\t\tbefore upgrading or removing InterScan eManager.\n"
		printf "\n"
		printf "\n"
		
		Exit
	fi
}

#---------------------------------------------------------
#  RemoveConfirmation
#---------------------------------------------------------
RemoveConfirmation ()
{
	
	if [ -f /etc/iscan_plugins/em.dsc ]
	then
		IsInterScanRunning
		
   		while true ; do

		Header

		printf "\n"
		printf "\n"
		printf "\t\tDo you really want to remove InterScan eManager?[y,n] ";read ANS

		#

		if [ "$ANS" = y -o "$ANS" = Y ]
		then
			#printf "YES"
			return 1
		elif [ "$ANS" = n -o "$ANS" = N ]
		then
			#printf "NO"
			return 0
		else
			printf "${BELL}\t\t"
		fi

		done

	else
		Header
		printf "\n"
		printf "\n"
		printf "\t\tNothing to remove.\n\n"
		
		Exit
	fi	    
}

#---------------------------------------------------------
#  KeepINI
#---------------------------------------------------------
KeepINI ()
{
	while true ; do
		Header
	
		printf "\n"
		printf "\n"
		printf "\t\tDo you want to keep the existing eManager rule files?[y,n] ";read KEEP
	
		if [ "$KEEP" = y -o "$KEEP" = Y ]
		then
		
			BackupConfig
			return
			
		elif [ "$KEEP" = n -o "$KEEP" = N ]
		then	
			return
		else
			printf "${BELL}\t\t"
		fi
	
	done
}

#---------------------------------------------------------
#  RemovePackage
#---------------------------------------------------------
RemovePackage()
{
	if [ -f /etc/iscan_plugins/em.dsc ]
	then

	emuxroot=`cat /etc/iscan_plugins/em.dsc | grep root_path | sed -e 's/=/ /' | awk '{print $2}'`
	
	#printf "$emuxroot/Pulg-Ins/EM\n"
	#Remove eManager Pakage

		
	#Ramove symbol link
	rm -rf $emuxroot/ISADMIN/IScan.adm/cgi-bin/EM
	rm -rf /etc/iscan_plugins/em.dsc
	
	#Remove eManager directory
	rm -rf $emuxroot/Plug-Ins/EM
	
	else

		Header
		printf "\n"
		printf "\n"
		printf "\t\tUnable to locate /etc/iscan_plugins/em.dsc\n\n"
		printf "\t\tUnable to remove package\n"
		
		Exit
	fi
}

#---------------------------------------------------------
#  UnSetCronTable
#---------------------------------------------------------
UnSetCronTable()
{
	crontab -l | grep -v dailylog | grep -v dailydownload > /tmp/rootcrontab
	crontab /tmp/rootcrontab
	rm -rf /tmp/rootcrontab
}

#=========================================================
#Remove_Process
#=========================================================
Remove_Process()
{
	RemoveConfirmation
	
	_Remove=$?
	
	if [ $_Remove -eq 1 ]
	then

		KeepINI

		Header
		printf "\n"
		printf "\n"
		printf "\t\tRemove all eManager packages and directories...\n"

		RemovePackage
		UnSetCronTable
		
		Header
		printf "\n"
		printf "\n"
		printf "\t\teManager has been removed.\n\n"
		
		Exit
	else
		#printf "eManager was not removed\n"
		return 1
	fi	
}

#=========================================================
#
#=========================================================
Exit()
{
	if [ -d $isuxroot/Plug-Ins/EM/cgi ] ; then
	  chmod -R 4550 $isuxroot/Plug-Ins/EM/cgi
        fi                
	printf "\n"
	Clean_Temp
}

#=========================================================
# print main menu
#=========================================================
Main_Menu()
{
    while true ; do
	Header
	printf "\t\tMain Menu\n"
	printf "\n"
	#Main_Status
	printf "\t1.\tInstall InterScan eManager.\n\n"
	printf "\t2.\tRemove InterScan eManager.\n\n"
	printf "\t3.\tExit.\n\n"
	printf "\t  \tSelect an option[   ]\b\b\b";read RESP
	case $RESP in
	1 ) Install_Process ;;
	2 ) Remove_Process ;;
	3 ) Exit 0 ;;
	* ) printf "${BELL}\t\tAn invalid choice was entered. Please try again."
	    read RESP ;;
	esac
    done
}

#=========================================================
# Main Program
#=========================================================
Main()
{
	trap Clean_Temp 1 2 15		
	SetPlatformDependVar

	CheckRoot
	
	if [ $? -ne 0 ]; then
	exit 1
	fi

	PreInstall

	CurVer=`$CheckVersion`
	
	if [ $CurVer = '<<_newer_version_already_installed_>>' ]
	then
	
		Header
		printf "\t\tA more current version of eManager was found\n"
		printf "\t\ton the system. Please use it to install/uninstall.\n"
		Exit 0
	
	else

		#if [ $CurVer = '3.5_XXX_Release_1027' -o $CurVer = '<<_the_same_version_but_old_build_>>' ]
		if [ $CurVer = '<<_older_than_version_3.7_installed_>>' -o $CurVer = '<<_version_3.7_installed_>>' -o $CurVer = '<<_the_same_version_but_old_build_>>' ]
		then
			Upgrade_Menu
		fi
	
		# Install new package, with restore confirm.
		Main_Menu
		
	fi
	
}

#=========================================================
#	Restore Configuration
#=========================================================
RestoreConfig ()
{
	emuxroot=`cat /etc/iscan_plugins/em.dsc | grep root_path | sed -e 's/=/ /' | awk '{print $2}'`
	#cur_dir=`pwd`
	
	if [ -f /etc/iscan_plugins/EM/contscan.ini ]
	then

		# Update serial or trial number of backup ini
		cp /etc/iscan_plugins/EM/contscan.ini /tmp/EMInstallFiles
		if [ $RightKey -eq -1 ]
		then
			/tmp/EMInstallFiles/WriteSNtoFile 'trial'
		elif [ $RightKey -eq 0 -o $RightKey -eq 2 ]
		then
			/tmp/EMInstallFiles/WriteSNtoFile 'serial' $SNRes
		fi
		SetHostLanguage
		cp /tmp/EMInstallFiles/contscan.ini /etc/iscan_plugins/EM

		cp /etc/iscan_plugins/EM/contscan.ini $emuxroot/Plug-Ins/EM 2> /dev/null
		
		cp /etc/iscan_plugins/EM/CsConfig.dat $emuxroot/Plug-Ins/EM 2> /dev/null
	
		cp /etc/iscan_plugins/EM/spamrule.txt $emuxroot/Plug-Ins/EM/spamrule 2> /dev/null
		cp /etc/iscan_plugins/EM/Bg_AntiSpam* $emuxroot/Plug-Ins/EM/spamrule 2> /dev/null
		cp /etc/iscan_plugins/EM/Trend* $emuxroot/Plug-Ins/EM/spamrule 2> /dev/null
		cp /etc/iscan_plugins/EM/iscan.log.* $emuxroot/Plug-Ins/EM 2> /dev/null

		[ -f /etc/iscan_plugins/EM/SFRule.txt ] && cp /etc/iscan_plugins/EM/SFRule.txt $emuxroot/Plug-Ins/EM/spamrule

		# restore dailydownload crontab entry
		crontab -l > /tmp/EMInstallFiles/roottab
		cat /etc/iscan_plugins/EM/rootcrontab >> /tmp/EMInstallFiles/roottab
		crontab /tmp/EMInstallFiles/roottab

	else
	
		#printf "\t\t$emuxroot/Plug-Ins/EM doesn't exist!\n\n"
		return 1
		
	fi
}

#---------------------------------------------------------
#  Backup Configuration
#---------------------------------------------------------
BackupConfig ()
{
	if [ -f /etc/iscan_plugins/em.dsc ]
	then
	
		emuxroot=`cat /etc/iscan_plugins/em.dsc | grep root_path | sed -e 's/=/ /' | awk '{print $2}'`

		[ -d /etc/iscan_plugins/EM ] || mkdir  /etc/iscan_plugins/EM
		[ -d /etc/iscan_plugins/EM/Backup ] || mkdir  /etc/iscan_plugins/EM/Backup
		[ -d /etc/iscan_plugins/EM/Backup/cgi ] || mkdir  /etc/iscan_plugins/EM/Backup/cgi
		
		cp $emuxroot/Plug-Ins/EM/contscan.ini /etc/iscan_plugins/EM 2> /dev/null
		cp $emuxroot/Plug-Ins/EM/CsConfig.dat /etc/iscan_plugins/EM 2> /dev/null
		
		cp $emuxroot/Plug-Ins/EM/spamrule/* /etc/iscan_plugins/EM 2> /dev/null
		cp $emuxroot/Plug-Ins/EM/* /etc/iscan_plugins/EM/Backup 2> /dev/null
		cp $emuxroot/Plug-Ins/EM/cgi/* /etc/iscan_plugins/EM/Backup/cgi 2> /dev/null
		rm -rf /etc/iscan_plugins/EM/iscan.log.*
		cp $emuxroot/Plug-Ins/EM/iscan.log.* /etc/iscan_plugins/EM 2> /dev/null
		
		chmod 600 /etc/iscan_plugins/EM/contscan.ini > /dev/null

		# backup dailydownload crontab entry
		crontab -l | grep dailydownload > /etc/iscan_plugins/EM/rootcrontab
	fi
	
	return 0
}

#=========================================================
# upgrade process
#=========================================================
Upgrade_Process()
{
	BackupFlag=$1

	if [ "$BackupFlag" = y -o "$BackupFlag" = Y ]
	then
		# Backup current configuration 
		BackupConfig
		# Upgrade backup files
		$UpgradeBackupConfig
	else
		# Remove old backup files
		rm -rf /etc/iscan_plugins/EM		
	fi

	if [ -f /etc/iscan_plugins/em.dsc ]
	then
		# Remove old eManager package			
		RemovePackage
		UnSetCronTable
	fi

	Install_Process

}

#=========================================================
# print upgrade menu
#=========================================================
Upgrade_Menu()
{
	Header
	printf "\t\tA previous version of eManager or a Rule File was found.\n"
	printf "\t\tYou can keep your current setting when upgrading eManager.\n\n"
	printf "\t\tWould you like to upgrade to version 3.8?[y,n] ";read READ
	
	if [ "$READ" = y -o "$READ" = Y ]
	then

		IsInterScanRunning

		KEEP=0
	    while [ "$KEEP" != y -a "$KEEP" != Y -a "$KEEP" != n -a "$KEEP" != N ] ; do
			Header
			printf "\n"
			printf "\n"
			printf "\t\tWould you like to keep the current Rule and Policy files?[y,n] ";read KEEP
			if [ "$KEEP" != y -a "$KEEP" != Y -a "$KEEP" != n -a "$KEEP" != N ]
			then
				printf "${BELL}\t\t"
			fi
	    done

		Header
		printf "\n"
		printf "\n"
		printf "\t\tUpgrading now...\n"

		Upgrade_Process $KEEP

		Exit 0
	fi
}

#=========================================================
#
#=========================================================
Main

